/* This example code is placed in the Public Domain. */

/* Parse a drawing written in s-expression into a compacted dom and
   "draw" it. Demonstrates how to load and process a dom. */

#include <stdlib.h>
#include <stdio.h>
#include "string.h"
#include <gensexpr/gsxl.h>

/* Return the idxth item on the list as a string (or NULL) */
static const char *arg_str(gsxl_node_t *node, int idx)
{
	node = gsxl_nth(node, idx);

	if (node == NULL)
		return NULL;
	return node->str;
}

/* Return the idxth item on the list as a double (or 0.0) */
static double arg_double(gsxl_node_t *node, int idx)
{
	node = gsxl_nth(node, idx);
	if (node == NULL)
		return 0.0;
	return strtod(node->str, NULL);
}

/* Expect pt children, print coords */
static void draw_poly_outline(const char *type, gsxl_node_t *node)
{
	gsxl_node_t *n;
	printf("draw polygon %s:", type);
	for(n = node; n != NULL; n = gsxl_next(n)) { /* points is a list */
		if ((n->str == NULL) || (strcmp(n->str, "pt") != 0)) /* deal with pt commands only */
			continue;
		printf(" %.2f;%.2f", arg_double(n, 1), arg_double(n, 2));
	}
	printf("\n");
}

/* Expect color, contour and hole children, print "drawing commands" */
static void draw_poly(gsxl_node_t *node)
{
	gsxl_node_t *n;
	for(n = node; n != NULL; n = gsxl_next(n)) {
		if (n->str == NULL) /* ignore empty command */
			continue;
		if (strcmp(n->str, "color") == 0)
			printf("Set color to %s\n", arg_str(n, 1));
		else if (strcmp(n->str, "contour") == 0)
			draw_poly_outline("contour", gsxl_children(n));
		else if (strcmp(n->str, "hole") == 0)
			draw_poly_outline("hole", gsxl_children(n));
	}
}

/* Draw the file recursively */
static void draw(gsxl_node_t *node)
{
	if (node->str == NULL)
		return;
	if (strcmp(node->str, "poly") == 0)
		draw_poly(gsxl_children(node));
}

int main(int argc, char *argv[])
{
	int c;
	gsx_parse_res_t res;
	gsxl_dom_t dom;

	/* Parse input */
	gsxl_init(&dom, gsxl_node_t);
	do {
		c = getchar();
	} while((res = gsxl_parse_char(&dom, c)) == GSX_RES_NEXT);

	if (res == GSX_RES_EOE) {
		/* compact and simplify the tree */
		gsxl_compact_tree(&dom);

		/* recursively draw the file */
		draw((gsxl_node_t *)dom.root);
	}

	/* clean up */
	gsxl_uninit(&dom);
	return 0;
}
