/*
Copyright (c) 2016 Tibor 'Igor2' Palinkas
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:
1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.
3. Neither the name of the Author nor the names of contributors
   may be used to endorse or promote products derived from this software
   without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.

Contact: gensexpr {at} igor2.repo.hu
Project page: http://repo.hu/projects/gensexpr
*/

/* CLI tester for the event parser */
#include <stdlib.h>
#include <stdio.h>
#include <gensexpr/gsx_parse.h>

void event(gsx_parse_t *ctx, gsx_parse_event_t ev, const char *data)
{
	printf("% 3d ", ctx->depth);
	switch(ev) {
		case GSX_EV_OPEN:  printf("open  "); break;
		case GSX_EV_CLOSE: printf("close "); break;
		case GSX_EV_ATOM:  printf("atom  "); break;
		case GSX_EV_ERROR: printf("error "); break;
	}
	printf("'%s'\n", data);
}

int main(int argc, char *argv[])
{
	int c;
	gsx_parse_res_t res;
	gsx_parse_t ctx;

	gsx_parse_init(&ctx);
	ctx.cb = event;
	ctx.line_comment_char = '#';
	do {
		c = getchar();
	} while((res = gsx_parse_char(&ctx, c)) == GSX_RES_NEXT);
	gsx_parse_uninit(&ctx);

	switch(res) {
		case GSX_RES_NEXT: abort(); /* can't happen */
		case GSX_RES_EOE: printf("<end-of-expression>\n"); break;
		case GSX_RES_ERROR: printf("<error>\n"); break;
	}

	return 0;
}